package com.emato.cuspay.support.msg.resp;

import com.google.common.base.Strings;
import com.google.common.collect.Lists;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 回执消息结构
 *
 * @author Scott Chen
 * @date 2017/4/20
 */
public class ResponseMessage {


    /**
     * 消息码
     */
    private String code;

    /**
     * 消息
     */
    private String msg;

    /**
     * 回执业务数据
     */
    private Object data;

    private ResponseMessage() {
    }

    public String getCode() {
        return code;
    }

    public String getMsg() {
        return msg;
    }

    public Object getData() {
        return data;
    }

    public static boolean isSuccess(ResponseMessage message) {
        return message.getCode() == null ? false : message.getCode().equals(ResponseStatus.SUCCESS.getItem()) ? true : false;
    }

    public static Builder builder() {
        return new Builder();
    }
    public static Builder builder(String code, String msg) {
        return new Builder(code, msg);
    }
    public static Builder builder(String code, String msg, ResponseMessageData responseMessageData) {
        return new Builder(code, msg, responseMessageData);
    }
    public static Builder builder(String code, String msg, List list) {
        return new Builder(code, msg, list);
    }
    public static <T> Builder builder(T t,  Class<T> type) {
        return new Builder(ResponseStatus.SUCCESS.getItem(), ResponseStatus.SUCCESS.getItemName(), t, type);
    }
    public static <T> Builder builder(String msg, T t,  Class<T> type) {
        return new Builder(ResponseStatus.SUCCESS.getItem(), msg, t, type);
    }
    public static <T> Builder builder(String code, String msg, T t,  Class<T> type) {
        return new Builder(code, msg, t, type);
    }


    //---------- 直接返回结果 ----------
    //成功
    public static ResponseMessage success() {
        return new Builder().build();
    }
    public static ResponseMessage success(String code, String msg) {
        return new Builder(code, msg).build();
    }
    public static ResponseMessage success(String msg) {
        return new Builder(ResponseStatus.SUCCESS.getItem(), msg).build();
    }

    public static ResponseMessage success(Map map) {
        return new Builder(ResponseStatus.SUCCESS.getItem(), ResponseStatus.SUCCESS.getItemName(), map).build();
    }
    public static ResponseMessage success(String msg, Map map) {
        return new Builder(ResponseStatus.SUCCESS.getItem(), msg, map).build();
    }
    public static ResponseMessage success(List rows) {
        return new Builder(ResponseStatus.SUCCESS.getItem(), ResponseStatus.SUCCESS.getItemName(), rows).build();
    }
    public static ResponseMessage success(String msg, List rows) {
        return new Builder(ResponseStatus.SUCCESS.getItem(), msg, rows).build();
    }
    public static ResponseMessage success(String code, String msg, List rows) {
        return new Builder(code, msg, rows).build();
    }
    public static <T> ResponseMessage success(T t,  Class<T> type) {
        return new Builder(ResponseStatus.SUCCESS.getItem(), ResponseStatus.SUCCESS.getItemName(), t, type).build();
    }
    public static <T> ResponseMessage success(String msg, T t,  Class<T> type) {
        return new Builder(ResponseStatus.SUCCESS.getItem(), msg, t, type).build();
    }
    public static <T> ResponseMessage success(String code, String msg, T t,  Class<T> type) {
        return new Builder(code, msg, t, type).build();
    }

    //错误
    public static ResponseMessage error() {
        return new Builder(ResponseStatus.ERROR.getItem(), ResponseStatus.ERROR.getItemName()).build();
    }
    public static ResponseMessage error(String code, String msg) {
        return new Builder(code, msg).build();
    }
    public static ResponseMessage error(String msg) {
        return new Builder(ResponseStatus.ERROR.getItem(), msg).build();
    }
    public static ResponseMessage error(Map map) {
        return new Builder(ResponseStatus.ERROR.getItem(), ResponseStatus.ERROR.getItemName(), map).build();
    }
    public static ResponseMessage error(String msg, Map map) {
        return new Builder(ResponseStatus.ERROR.getItem(), msg, map).build();
    }
    public static ResponseMessage error(List rows) {
        return new Builder(ResponseStatus.ERROR.getItem(), ResponseStatus.ERROR.getItemName(), rows).build();
    }
    public static ResponseMessage error(String msg, List rows) {
        return new Builder(ResponseStatus.ERROR.getItem(), msg, rows).build();
    }
    public static ResponseMessage error(String code, String msg, List rows) {
        return new Builder(code, msg, rows).build();
    }

    //失败
    public static ResponseMessage failed() {
        return new Builder(ResponseStatus.FAILURE.getItem(), ResponseStatus.FAILURE.getItemName()).build();
    }
    public static ResponseMessage failed(String code, String msg) {
        return new Builder(code, msg).build();
    }
    public static ResponseMessage failed(String msg) {
        return new Builder(ResponseStatus.FAILURE.getItem(), msg).build();
    }

    public static ResponseMessage failed(Map map) {
        return new Builder(ResponseStatus.ERROR.getItem(), ResponseStatus.ERROR.getItemName(), map).build();
    }
    public static ResponseMessage failed(String msg, Map map) {
        return new Builder(ResponseStatus.ERROR.getItem(), msg, map).build();
    }
    public static ResponseMessage failed(List rows) {
        return new Builder(ResponseStatus.FAILURE.getItem(), ResponseStatus.FAILURE.getItemName(), rows).build();
    }
    public static ResponseMessage failed(String msg, List rows) {
        return new Builder(ResponseStatus.ERROR.getItem(), msg, rows).build();
    }
    public static ResponseMessage failed(String code, String msg, List rows) {
        return new Builder(code, msg, rows).build();
    }

    /**
     * 消息构建器类
     */
    public static class Builder {
        private String code;
        private String msg;
        private Object data;

        public Builder() {
            this.code = ResponseStatus.SUCCESS.getItem();
            this.msg = ResponseStatus.SUCCESS.getItemName();
            this.data = Lists.newArrayList();
        }

        public Builder(String code, String msg) {
            this.code = Strings.isNullOrEmpty(code) ? ResponseStatus.SUCCESS.getItem() : code;
            this.msg = Strings.isNullOrEmpty(msg) ? ResponseStatus.SUCCESS.getItemName() : msg;
            this.data = Lists.newArrayList();
        }

        public Builder(String code, String msg, ResponseMessageData responseMessageData) {
            this.code = Strings.isNullOrEmpty(code) ? ResponseStatus.SUCCESS.getItem() : code;
            this.msg = Strings.isNullOrEmpty(msg) ? ResponseStatus.SUCCESS.getItemName() : msg;
            this.data = Lists.newArrayList();
        }

        public Builder(String code, String msg, Map map) {
            this.code = Strings.isNullOrEmpty(code) ? ResponseStatus.SUCCESS.getItem() : code;
            this.msg = Strings.isNullOrEmpty(msg) ? ResponseStatus.SUCCESS.getItemName() : msg;
            this.data = Lists.newArrayList();
        }

        public Builder(String code, String msg, List rows) {
            this.code = Strings.isNullOrEmpty(code) ? ResponseStatus.SUCCESS.getItem() : code;
            this.msg = Strings.isNullOrEmpty(msg) ? ResponseStatus.SUCCESS.getItemName() : msg;
            this.data = Lists.newArrayList();
        }

        public <T> Builder(T t, Class<T> type) {
            this.code = ResponseStatus.SUCCESS.getItem();
            this.msg = ResponseStatus.SUCCESS.getItemName();
            this.data = Lists.newArrayList();
        }

        public <T> Builder(String msg, T t, Class<T> type) {
            this.code = ResponseStatus.SUCCESS.getItem();
            this.msg = Strings.isNullOrEmpty(msg) ? ResponseStatus.SUCCESS.getItemName() : msg;
            this.data = Lists.newArrayList();
        }

        public <T> Builder(String code, String msg, T t, Class<T> type) {
            this.code = Strings.isNullOrEmpty(code) ? ResponseStatus.SUCCESS.getItem() : code;
            this.msg = Strings.isNullOrEmpty(msg) ? ResponseStatus.SUCCESS.getItemName() : msg;
            this.data = Lists.newArrayList();
        }

        public Builder setCode(String code) {
            this.code = code;
            return this;
        }

        public Builder setMsg(String msg) {
            this.msg = msg;
            return this;
        }

        public Builder setData(Object data) {
            this.data = data;
            return this;
        }

        public ResponseMessage build() {
            ResponseMessage message = new ResponseMessage();
            message.code = this.code;
            message.msg = this.msg;
            message.data = this.data;
            return message;
        }

    }
}

